<?PHP
###########################################################################
# 
# File:            taloLoggerGraph_database.inc (PHP)
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0c
#
# Date:            03.12.2015
#
# Classes:         DBConnection
#                    MySQLDBConnection
#                    PostgreSQLDBConnection
#                    SQLiteDBConnection
#                   
# Description:     House automation data logger graphing application for PHP. 
# 
#                  Implementing database connection class for the package.
#
# Requirements:    PHP 5.1 or newer (www.php.net)
#
#                  Suitable PDO database support for PHP or mysql or postgresql or sqlite interface for PHP.
#
# Version history: ** 05.11.2010 v1.0b (Olli Lammi) **
#                  Database operations implemented in class
#                  to provide interoperability with systems
#                  not having PDO. 
#
#                  ** 03.12.2015 v1.0c (Olli Lammi) **
#                  Support for PostgreSQL.
#
###########################################################################

class DBConnection {
    public function isOpen() {
        return 0;
    }

    public function close() {
    }
    
    public function query($sql) {
        return 0;
    }
    
    public function fetchRow() {
        return null;
    }
    
    public function freeResult() {
    }
}

class MySQLDBConnection extends DBConnection {
    private $conn = null;
    private $type = 'none';
    private $result = null;

    public function __construct($host, $dbname, $username, $passwd) {
        if (class_exists('PDO') && in_array('mysql', PDO::getAvailableDrivers())) {
            $this->type = 'PDO';
            try {
                $this->conn = new PDO("mysql:host=" . $host . ";dbname=" . $dbname, $username, $passwd);
            }  
            catch(PDOException $e) {
                $this->conn = null;
            }            
        }
        else if (function_exists('mysql_connect')) {
            $this->type = 'interface';
            $this->conn = mysql_connect($host, $username, $passwd);
            if ($this->conn) mysql_select_db($dbname, $this->conn);
        }
        else {
            error_log("ERROR: Cannot find driver support for MySQL databases.");
            $this->type = 'none';
            $this->conn = null;
        }   
    }
    
    public function __destruct() {
        if ($this->conn) $this->close();
    }
    
    public function isOpen() {
        if ($this->conn) return 1;
        return 0;
    }

    public function close() {
        $this->freeResult();
        if ($this->conn) {
            if ($this->type == 'interface') {
                mysql_close($this->conn);
            }
            $this->conn = null;
        }
    }
    
    public function query($sql) {
        if (!$this->conn) return 0;
        $this->freeResult();
        if ($this->type == 'PDO') {
            $this->result = $this->conn->query($sql);
        }
        else if ($this->type == 'interface') {
            $this->result = mysql_query($sql, $this->conn);
        }
        if ($this->result) return 1;
        return 0;
    }
    
    public function fetchRow() {
        if (!$this->result) return null;
        
        if ($this->type == 'PDO') {
            return $this->result->fetch(PDO::FETCH_ASSOC);
        }
        else if ($this->type == 'interface') {
        	return mysql_fetch_array($this->result);
        }
        
        return null;
    }
    
    public function freeResult() {
        if (!$this->result) return;
        if ($this->type == 'interface') {
            mysql_free_result($this->result);
        }
        $this->result = null;
    }   
}

class PostgreSQLDBConnection extends DBConnection {
	private $conn = null;
	private $type = 'none';
	private $result = null;

	public function __construct($host, $dbname, $username, $passwd) {
		if (class_exists('PDO') && in_array('pgsql', PDO::getAvailableDrivers())) {
			$this->type = 'PDO';
			try {
				$this->conn = new PDO("pgsql:host=" . $host . ";dbname=" . $dbname, $username, $passwd);
			}
			catch(PDOException $e) {
				$this->conn = null;
			}
		}
		else if (function_exists('pg_connect')) {
			$this->type = 'interface';
			$this->conn = pg_connect("host=" . $host . " dbname=" . $dbname . " user=" . $username . " password=" . $passwd);
		}
		else {
			error_log("ERROR: Cannot find driver support for PostgreSQL databases.");
			$this->type = 'none';
			$this->conn = null;
		}
	}

	public function __destruct() {
		if ($this->conn) $this->close();
	}

	public function isOpen() {
		if ($this->conn) return 1;
		return 0;
	}

	public function close() {
		$this->freeResult();
		if ($this->conn) {
			if ($this->type == 'interface') {
				pg_close($this->conn);
			}
			$this->conn = null;
		}
	}

	public function query($sql) {
		if (!$this->conn) return 0;
		$this->freeResult();
		if ($this->type == 'PDO') {
			$this->result = $this->conn->query($sql);
		}
		else if ($this->type == 'interface') {
			$this->result = pg_query($this->conn, $sql);
		}
		if ($this->result) return 1;
		return 0;
	}

	public function fetchRow() {
		if (!$this->result) return null;

		if ($this->type == 'PDO') {
			return $this->result->fetch(PDO::FETCH_ASSOC);
		}
		else if ($this->type == 'interface') {
			return pg_fetch_array($this->result, NULL, PGSQL_ASSOC);
		}

		return null;
	}

	public function freeResult() {
		if (!$this->result) return;
		if ($this->type == 'interface') {
			pg_free_result($this->result);
		}
		$this->result = null;
	}
}


class SQLiteDBConnection extends DBConnection {
    private $conn = null;
    private $type = 'none';
    private $result = null;

    public function __construct($dbname) {
        if (class_exists('PDO') && in_array('sqlite', PDO::getAvailableDrivers())) {
            $this->type = 'PDO';
            try {
                $this->conn = new PDO("sqlite:" . $dbname);
            }  
            catch(PDOException $e) {
                $this->conn = null;
            }            
        }
        else if (function_exists('sqlite_open')) {
            $this->type = 'interface';
            $this->conn = sqlite_open($dbname);
        }
        else {
            error_log("ERROR: Cannot find driver support for SQLite databases.");
            $this->type = 'none';
            $this->conn = null;
        }   
    }
    
    public function __destruct() {
        if ($this->conn) $this->close();
    }
    
    public function isOpen() {
        if ($this->conn) return 1;
        return 0;
    }

    public function close() {
        $this->freeResult();
        if ($this->conn) {
            if ($this->type == 'interface') {
                sqlite_close($this->conn);
            }
            $this->conn = null;
        }
    }
    
    public function query($sql) {
        if (!$this->conn) return 0;
        $this->freeResult();
        if ($this->type == 'PDO') {
            $this->result = $this->conn->query($sql);
        }
        else if ($this->type == 'interface') {
            $this->result = sqlite_query($this->conn, $sql);
        }
        if ($this->result) return 1;
        return 0;
    }
    
    public function fetchRow() {
        if (!$this->result) return null;
        
        if ($this->type == 'PDO') {
            return $this->result->fetch(PDO::FETCH_ASSOC);
        }
        else if ($this->type == 'interface') {
            return sqlite_fetch_array($this->result, SQLITE_ASSOC);
        }
        
        return null;
    }
    
    public function freeResult() {
        if (!$this->result) return;
        $this->result = null;
    }   
}
               
?>
